import sys
import time
import requests
import utils
import logging
import os
import docker
import rov_server_config

logging.basicConfig(level=logging.INFO,
                    format='%(asctime)s %(filename)s[line:%(lineno)d] %(levelname)s %(message)s',
                    datefmt='%a, %d %b %Y %H:%M:%S')

health_instances = []
ess_instances = []
remove_container_instances = []


def server_health_check(client, instance_id):
    """
    服务健康检查
    :param client: 客户端连接
    :param instance_id: instanceId
    :return:
    """
    global health_instances
    ip_address = utils.get_ip_address(client=client, instance_id=instance_id)
    while True:
        health_check_url = f"http://{ip_address}:5001/healthcheck"
        try:
            http_code = requests.get(health_check_url).status_code
        except:
            logging.info("images is downloading")
            http_code = 0

        if http_code == 200:
            health_instances.append((instance_id, ip_address))
            break
        else:
            time.sleep(10)


def remove_container_image(client, instance_id, container_name):
    """
    移除旧容器并删除旧镜像
    :param client: 客户端连接
    :param instance_id: instanceId type-string
    :param container_name: 容器名称 type-string
    :return:
    """
    ip_address = utils.get_ip_address(client=client, instance_id=instance_id)
    logging.info(f"服务器信息：{instance_id}/{ip_address}")
    client = docker.DockerClient(base_url=f'tcp://{ip_address}:2375', timeout=60)
    # 移除旧的容器
    container_remove_retry = 3
    i = 0
    while True:
        if i >= container_remove_retry:
            logging.error(f"容器不存在或者无法删除当前容器, instance = {instance_id}/{ip_address}")
            sys.exit()
        try:
            container_id = client.containers.get(container_name)
            container_id.remove(force=True)
            break
        except Exception as e:
            i += 1

    # 删除旧镜像
    images_remove_retry = 3
    j = 0
    while True:
        if j >= images_remove_retry:
            logging.error(f"镜像不存在，无法获取到镜像ID, instance = {instance_id}/{ip_address}")
            sys.exit()
        try:
            images = client.images.list()
            for image in images:
                client.images.remove(force=True, image=image.tags[0])
                time.sleep(2)
            global remove_container_instances
            remove_container_instances.append(instance_id)
            break
        except Exception as e:
            i += 1


def update_instance(create_client, slb_client, instance_id, version):
    """
    线上机器更新
    :param create_client:
    :param slb_client: slb客户端连接
    :param instance_id: instanceId
    :param version: 版本标记
    :return:
    """
    logging.info(f"update instance: {instance_id}")
    # 1. 摘流量
    utils.set_instance_weight_process(client=slb_client,
                                      slb_id=rov_server_config.slb_id,
                                      instance_id_list=[instance_id],
                                      weight_list=[(0, 60)])
    logging.info(f"set weight with 0 finished, instance: {instance_id}")
    # 2. 异步移除旧容器并删除旧镜像
    global remove_container_instances
    remove_container_instances = []
    container_name = 'rov-server'
    remove_container_image(slb_client, instance_id, container_name)
    logging.info(f"remove container & images finished, instances: {remove_container_instances}, "
                 f"count: {len(remove_container_instances)}")
    if len(remove_container_instances) == 0:
        logging.error(f"remove container image failed| ")
        sys.exit()
    # 3. 发送启动脚本到机器上
    utils.send_file_to_ecs(client=create_client, instance_id_list=[instance_id], **rov_server_config.start_sh)
    logging.info(f"send start shell file finished, instance: {instance_id}")
    # 4. 启动服务
    server_start_sh = os.path.join(rov_server_config.start_sh['target_dir'], rov_server_config.start_sh['name'])
    server_start_commend = f"sh {server_start_sh} {version}"
    utils.run_command(client=create_client, instance_ids=[instance_id], command=server_start_commend)
    # 5. 探活
    global health_instances
    health_instances = []
    server_health_check(slb_client, instance_id)
    logging.info(f"health instances: {health_instances}, count: {len(health_instances)}")
    # 6. 挂载流量
    if len(health_instances) == 0:
        logging.info(f"health instances: {health_instances}, count: {len(health_instances)}")
        sys.exit()
    # 机器探活成功
    time.sleep(60)
    utils.add_backend_servers(client=slb_client, slb_id=rov_server_config.slb_id, instances=health_instances)
    health_instance_ids = [instance_id for instance_id, _ in health_instances]
    add_weight_list = [(10, 30), (20, 20), (40, 10), (60, 10), (80, 10), (100, 10)]
    utils.set_instance_weight_process(client=slb_client,
                                      slb_id=rov_server_config.slb_id,
                                      instance_id_list=health_instance_ids,
                                      weight_list=add_weight_list)
    logging.info(f"finished instances: {health_instances}, count: {len(health_instances)}")


def main():
    try:
        # 1. 获取回滚的版本
        version = sys.argv[1]
        logging.info(f"rollback version: {version}")

        # 2. 获取指定回滚的机器列表
        param = sys.argv[2]
        instance_id_list = [instance_id for instance_id in param.replace(" ", "").split(',')]
        logging.info(f"rollback instances count: {len(instance_id_list)}")
        logging.info(f"rollback instances : {instance_id_list}")

        slb_client = utils.connect_client(
            access_key_id=rov_server_config.slb_client_params['access_key_id'],
            access_key_secret=rov_server_config.slb_client_params['access_key_secret'],
            region_id=rov_server_config.slb_client_params['region_id']
                                          )
        create_client = utils.connect_client(
            access_key_id=rov_server_config.create_client_params['access_key_id'],
            access_key_secret=rov_server_config.create_client_params['access_key_secret'],
            region_id=rov_server_config.create_client_params['region_id']
        )

        # 3. 回滚
        logging.info(f"rollback instances start ...")
        for instance_id in instance_id_list:
            logging.info(f"rollback instance_id: {instance_id}")
            update_instance(create_client=create_client,
                            slb_client=slb_client,
                            instance_id=instance_id,
                            version=version)
        logging.info(f"rollback instances end!")

    except Exception as e:
        logging.error(e)
        sys.exit()


if __name__ == '__main__':
    main()
